from .classifier import BasicTweetClassifier
from .commands import BasicTwitterCommands
from .preprocessor import ProcessTokens
from .questions import BasicTwitterQuestion

__all__ = [
    "BasicTweetClassifier",
    "BasicTwitterCommands",
    "ProcessTokens",
    "BasicTwitterQuestion",
]



from abc import ABC, abstractmethod

from pydantic import BaseModel


class ContextT(ABC, BaseModel):
    @abstractmethod
    async def load_extra_context(self) -> str:
        """Load extra context for the LLM"""



from .effects import Effect
from .executor import Executor
from .llms import Classifier, ClassifierResponse, ContextT
from .processor import PreProcess, Processor
from .sources import SourceT
from .user import UserManagerT

__all__ = [
    "Classifier",
    "ClassifierResponse",
    "ContextT",
    "Effect",
    "Executor",
    "PreProcess",
    "Processor",
    "SourceT",
    "UserManagerT",
]



import json
from typing import Annotated, Callable, Literal, cast

from eth_rpc.utils import to_checksum
from pydantic import BaseModel, Field
from typing_extensions import Doc

from dowse.interfaces import UserManagerT
from dowse.logger import logger
from dowse.models.commands import SwapArgs, TransferArgs
from dowse.tools import convert_decimal_eth_to_wei, get_token_address_tool
from dowse.tools.best_route.kyber import Quote
from dowse.tools.best_route.kyber import get_quote as get_kyber_quote

from .user import BasicUserManager


class CommandRequest(BaseModel):
    command: Literal["swap", "transfer"]
    args: SwapArgs | TransferArgs = Field(
        description="The arguments for the command, either swap args or transfer args",
    )


class CommandTools(BaseModel):
    username_to_address: dict[str, str] = Field(default_factory=dict)
    user_manager: UserManagerT = Field(default_factory=BasicUserManager)

    async def execute_command(self, command: CommandRequest) -> None:
        if command.command == "swap":
            swap_args = cast(SwapArgs, command.args)
            await self.execute_swap(swap_args)
        elif command.command == "transfer":
            transfer_args = cast(TransferArgs, command.args)
            await self.execute_transfer(transfer_args)

    async def make_swap_command(
        self,
        token_in: Annotated[
            str,
            Doc(
                "The token to swap from.  Must be a token symbol, token address, or ETH."
            ),
        ],
        token_out: Annotated[
            str, Doc("The token to swap to.  Must be a token symbol or ETH")
        ],
        amount: Annotated[int, Doc("The amount to swap")],
        recipient: Annotated[
            str,
            Doc(
                "The recipient of the swap.  Can be a twitter username or an ethereum address.  "
                "If it starts with @ it's a username, don't add 0x to the front of it"
            ),
        ],
        chain_id: Annotated[int, Doc("The chain id")] = 8453,
    ) -> str:
        """Convert the swap arguments to the structured syntax for a swap"""
        slippage = 1
        logger.debug(
            "Tool Call: make_swap_command (%s, %s, %s, %s, %s, %s)",
            token_in,
            token_out,
            amount,
            slippage,
            chain_id,
            recipient,
        )

        input_token = await get_token_address_tool(token_in)
        output_token = await get_token_address_tool(token_out)

        chain_id = cast(Literal[1, 8453, 42161, 10, 137, 43114], chain_id)
        quote: Quote = await get_kyber_quote(
            token_in=to_checksum(input_token),
            token_out=to_checksum(output_token),
            amount=amount,
            chain_id=chain_id,
        )

        return json.dumps(
            {
                "amount_out": str(int(quote.amount_out)),
                "aggregator": quote.aggregator,
                "input_token_symbol": token_in,
                "input_token_address": input_token,
                "output_token_symbol": token_out,
                "output_token_address": output_token,
            }
        )

    async def make_transfer_command(
        self,
        token: Annotated[str, Doc("The token to transfer")],
        amount: Annotated[int, Doc("The amount to transfer")],
        sender: Annotated[str, Doc("The sender of the transfer")],
        recipient: Annotated[str, Doc("The recipient of the transfer")],
    ) -> str:
        """Create a transfer command, returning the output that would be used to make the transfer"""
        logger.debug(
            f"Tool Call: make_transfer_command ({token}, {amount}, {sender}, {recipient})"
        )

        token_address = await get_token_address_tool(token)
        return json.dumps(
            {
                "token_symbol": token,
                "token_address": token_address,
                "sender": sender,
                "recipient": recipient,
                "value": str(int(amount)),
            }
        )

    async def get_user_address_helper(
        self,
        username: Annotated[str, Doc("The username to get the address for")],
    ) -> str:
        """Get the address for a username"""
        if not self.username_to_address.get(username):
            address = await self.user_manager.get_user_address(username)
            self.username_to_address[username] = address
        return self.username_to_address[username]

    async def execute_swap(self, command: SwapArgs) -> None:
        """Execute a swap"""
        raise NotImplementedError("Swap not implemented")

    async def execute_transfer(self, command: TransferArgs) -> None:
        """Execute a transfer"""
        raise NotImplementedError("Transfer not implemented")

    def tools(self) -> list[Callable]:
        return [
            convert_decimal_eth_to_wei,
            self.make_swap_command,
            self.make_transfer_command,
        ]


Tools = CommandTools()



from typing import Annotated

from pydantic import BaseModel
from typing_extensions import Doc


class TransferArgs(BaseModel):
    token_symbol: Annotated[str | None, Doc("The symbol of the token to transfer")]
    token_address: Annotated[str, Doc("The address of the token to transfer")]
    amount: Annotated[int, Doc("The amount of the token to transfer")]

    recipient: Annotated[str, Doc("The recipient of the transfer")]
    sender: Annotated[str, Doc("The sender of the transfer")]



from emp_agents.providers import OpenAIModelType, OpenAIProvider
from pydantic import BaseModel

from dowse.interfaces.executor import Executor
from dowse.models import Tweet


class Response(BaseModel):
    response: str


BasicTwitterQuestion = Executor[Tweet, Tweet, Response](
    provider=OpenAIProvider(
        default_model=OpenAIModelType.gpt4o,
    ),
)



import os

import httpx
from eth_typing import HexAddress, HexStr
from pydantic import BaseModel, Field

from dowse.logger import logger

QUICKNODE_ENDPOINT = os.environ["QUICKNODE_ENDPOINT"]


class Quote(BaseModel):
    token_in: HexAddress = Field(alias="tokenIn")
    token_out: HexAddress = Field(alias="tokenOut")
    amount_in: int = Field(alias="amountIn")
    amount_out: int = Field(alias="amountOut")
    gas: int
    amount_out_min: float = Field(alias="amountOutMin")
    aggregator: str


class Quotes(BaseModel):
    quotes: list[Quote]


class SwapResponse(BaseModel):
    from_: HexAddress = Field(alias="from")
    to: HexAddress
    data: str
    value: int
    chain_id: int = Field(alias="chainId")
    aggregator: str


async def get_quote(
    token_out: HexAddress,
    amount: int,
    slippage: float,
    chain_id: int,
    recipient: HexAddress,
    token_in: HexAddress,
):
    url = f"{QUICKNODE_ENDPOINT}/addon/688/quote"

    if token_in.lower() == "eth":
        token_in = HexAddress(HexStr("0x4200000000000000000000000000000000000006"))

    tries = 0
    while tries < 3:
        tries += 1
        try:
            async with httpx.AsyncClient() as client:
                logger.debug(
                    f"GET QUOTE: {token_in}, {token_out}, {amount}, {slippage}, {chain_id}, {recipient}"
                )
                response = await client.post(
                    url,
                    json={
                        "tokenIn": token_in,
                        "tokenOut": token_out,
                        "amountIn": amount,
                        "slippage": slippage,
                        "chainId": chain_id,
                        "recipient": recipient,
                    },
                    headers={
                        "Content-Type": "application/json",
                    },
                    timeout=20,
                )
                data = response.json()
                if data.get("message") == "Internal Server Error":
                    raise Exception("Internal Server Error")
                return data
        except Exception as e:
            logger.error("ERROR GETTING QUOTE", e)

    raise ValueError("Failed to get quote.  Most likely an API Limit")


async def swap(
    token_in: HexAddress,
    token_out: HexAddress,
    amount: int,
    slippage: float,
    chain_id: int,
    recipient: HexAddress,
    aggregator: str,
):
    url = f"{QUICKNODE_ENDPOINT}/addon/688/swap"
    async with httpx.AsyncClient() as client:
        response = await client.post(
            url,
            json={
                "aggregator": aggregator,
                "tokenIn": token_in,
                "tokenOut": token_out,
                "amountIn": amount,
                "slippage": slippage,
                "chainId": chain_id,
                "recipient": recipient,
            },
            headers={
                "Content-Type": "application/json",
            },
        )
        return SwapResponse(**response.json())



import json
from typing import Callable

from pydantic import BaseModel

from dowse.interfaces.processor.base import Processor
from dowse.models import Tweet
from dowse.tools import convert_dollar_amount_to_eth, get_token_address_tool


class FormattedCommand(BaseModel):
    content: str
    caller: str


class ProcessTokens(Processor[Tweet, FormattedCommand]):
    tools: list[Callable] = [
        get_token_address_tool,
        convert_dollar_amount_to_eth,
    ]

    async def to_string(self, command: Tweet) -> str:
        return json.dumps({"caller": command.creator_name, "content": command.content})



from abc import ABC
from typing import Any, Generic, Literal, TypeVar, get_args

from emp_agents import AgentBase
from emp_agents.models import Provider
from emp_agents.providers import OpenAIModelType, OpenAIProvider
from pydantic import BaseModel, PrivateAttr

from ..example_loader import ExampleLoader
from ..prompt_loader import PromptLoader

InputType = TypeVar("InputType", bound=BaseModel)
Classifications = TypeVar(
    "Classifications",
)

T = TypeVar("T")


class ClassifierResponse(BaseModel, Generic[T]):
    __name__ = "ClassifierResponse"
    classification: T


class Classifier(ABC, ExampleLoader, PromptLoader, Generic[InputType, Classifications]):
    prompt: str = (
        "classify the input into the response_format.  Respond with the classification and nothing else."
    )
    provider: Provider = OpenAIProvider(
        default_model=OpenAIModelType.gpt4o,
    )

    _agent: AgentBase = PrivateAttr()

    def __init_subclass__(cls, **kwargs):
        super().__init_subclass__(**kwargs)
        cls.load_examples()
        cls.load_prompt()

    def model_post_init(self, __context: Any) -> None:
        super().model_post_init(__context)

        self._agent = AgentBase(
            prompt=self.prompt,
            provider=self.provider,
        )
        if len(type(self).__pydantic_generic_metadata__["args"]) != 2:
            raise ValueError("Classifier must have input type and classifications set")

        if self._prompt is not None:
            self.prompt = self._prompt
        if self._examples:
            self.examples = self._examples

    async def classify(
        self,
        message: InputType,
    ) -> Classifications:
        (_, classifications) = type(self).__pydantic_generic_metadata__["args"]

        classification_type = get_args(get_args(Literal[*classifications])[0])[0]  # type: ignore

        response_type = ClassifierResponse[classification_type]  # type: ignore[valid-type]
        response_type.__name__ = "ClassifierResponse"

        response = await self._agent.answer(
            message.model_dump_json(),
            response_format=response_type,
        )
        return ClassifierResponse.model_validate_json(response).classification



# flake8: noqa

from emp_agents.models import AssistantMessage, ToolCall, ToolMessage, UserMessage

EXAMPLES = [
    [
        UserMessage(content='{"caller": "@user", "content": "swap $100 for AERO"}'),
        AssistantMessage(
            content="I will use my tools to get the token address for AERO",
            tool_calls=[
                ToolCall(
                    id="57d74922-3636-49b1-8109-fe25ca66ca19",
                    type="function",
                    function=ToolCall.Function(
                        name="get_token_address_tool",
                        arguments='{"symbol_or_token_address": "AERO"}',
                    ),
                ),
                ToolCall(
                    id="0b73576d-413a-4fb8-8fe4-99da789fb0e1",
                    type="function",
                    function=ToolCall.Function(
                        name="convert_dollar_amount_to_eth",
                        arguments='{"amount": "123123.1231"}',
                    ),
                ),
            ],
        ),
        ToolMessage(
            tool_call_id="57d74922-3636-49b1-8109-fe25ca66ca19",
            content="0x940181a94A35A4569E4529A3CDfB74e38FD98631",
        ),
        ToolMessage(
            tool_call_id="0b73576d-413a-4fb8-8fe4-99da789fb0e1", content="2.9649"
        ),
        AssistantMessage(
            content='{"content": {"caller": "@user", "user_request": "swap 123123.1231 ETH (0x4200000000000000000000000000000000000006) for $AERO (0x940181a94A35A4569E4529A3CDfB74e38FD98631)", "error_message": null}}'
        ),
        UserMessage(
            content="Ok Great, now lets ignore these values but use this as the general structure of how the processing should go",
        ),
        AssistantMessage(content="I understand."),
    ],
]



from dowse.interfaces.sources import SourceT
from dowse.models import Tweet


class TwitterMock(SourceT[Tweet]):
    async def get_data(self) -> list[Tweet]:
        """Load tweets to be parsed for commands/questions"""
        return [
            Tweet(
                id=1890118705016877145,
                content="""#simmi buy me $10 of $BNKR, swap half of it to $TN100x, and then send my $TN100x to
            @696_eth
             along with 0.0001 ETH for gas""",
                creator_id=1414021381298089984,
                creator_name="@0xDeployer",
            ),
            Tweet(
                id=1684298214198108160,
                content="""Why do you continue to ignore bitcoin and lightning?

what "crypto" is a better money transmission protocol and why?""",
                creator_id=12,
                creator_name="@jack",
            ),
        ]

    async def handle(self, data: Tweet) -> None:
        """Respond to a tweet"""
        print("RESPONDING TO TWEET:", data)

    async def has_handled(
        self,
        data: Tweet,
    ) -> bool:
        """Check if the bot has already responded to a tweet"""
        return False

    async def mark_as_handled(
        self,
        data: Tweet,
    ) -> None:
        """Mark a tweet as responded to so you don't respond to it twice"""



from typing import Any, Generic, TypeVar

from pydantic import BaseModel, Field

T = TypeVar("T")


class AgentMessage(BaseModel, Generic[T]):
    content: T | None = Field(
        description="The content of the message.  If the agent is unable to complete the task, set this to None",
    )
    error_message: str | None = Field(
        description="An optional error message.  Only provide this if content is None",
    )

    def model_post_init(self, __context: Any) -> None:
        if self.content is None and self.error_message is None:
            raise ValueError("content or error_message must be set")

    @property
    def is_error(self) -> bool:
        return self.error_message is not None



from eth_typeshed.erc20 import ERC20


async def convert_token_amount_to_wei(amount: str, token_address: str) -> str:
    """
    convert a decimal token amount to wei
    """
    token = ERC20(address=token_address)
    decimals = await token.decimals()
    return str(int(float(amount) * 10**decimals))



from typing import Annotated

from pydantic import BaseModel
from typing_extensions import Doc


class SwapArgs(BaseModel):
    token_in_symbol: Annotated[str, Doc("The symbol of the token to swap from")]
    token_out_symbol: Annotated[str, Doc("The symbol of the token to swap to")]
    token_in_address: Annotated[str, Doc("The address of the token to swap from")]
    token_out_address: Annotated[str, Doc("The address of the token to swap to")]
    amount_in: Annotated[int, Doc("The amount of tokens to swap")]
    amount_out: Annotated[int, Doc("The amount of tokens to receive")]
    slippage: Annotated[float, Doc("The slippage tolerance")]

    sender: Annotated[str, Doc("The sender of the transaction")]
    recipient: Annotated[str, Doc("The recipient of the transaction")]



from abc import ABC, abstractmethod
from typing import Generic, Sequence, TypeVar

from pydantic import BaseModel

T = TypeVar("T", bound=BaseModel)


class SourceT(ABC, BaseModel, Generic[T]):
    @abstractmethod
    async def get_data(self) -> Sequence[T]:
        """Load data to be parsed for commands/questions"""

    async def handle(self, data: T) -> None:
        """Handle the data"""

    async def has_handled(
        self,
        data: T,
    ) -> bool:
        """Check if the bot has already handled the data"""
        return False

    async def mark_as_handled(
        self,
        data: T,
    ) -> None:
        """Mark a data as handled so you don't handle it twice"""



from pathlib import Path
from typing import ClassVar

from emp_agents.models import Message
from pydantic import BaseModel, Field, PrivateAttr


class ExampleLoader(BaseModel):
    _examples: list[list[Message]] = PrivateAttr(default_factory=list)
    examples: list[list[Message]] = Field(default_factory=list)
    _path: ClassVar[Path]

    @classmethod
    def load_examples(cls) -> None:
        import inspect

        stack = inspect.stack()

        parent_path = Path(__file__).parent

        caller_frame = None
        for frame_info in stack:
            filename = frame_info.filename
            if filename.startswith("<frozen"):
                continue
            if Path(filename).is_relative_to(parent_path):
                continue
            if "pydantic" not in filename:
                caller_frame = frame_info
                break

        assert caller_frame is not None
        cls._path = Path(caller_frame.filename).parent

        examples_path = cls._path / "examples.py"
        if examples_path.exists():
            from importlib.util import module_from_spec, spec_from_file_location

            spec = spec_from_file_location("examples", examples_path)
            if spec and spec.loader:
                module = module_from_spec(spec)
                spec.loader.exec_module(module)
                if hasattr(module, "EXAMPLES"):
                    cls._examples = module.EXAMPLES



import logging
import sys

logger = logging.getLogger("dowse")

logger.setLevel(logging.INFO)
handler = logging.StreamHandler(sys.stdout)
handler.setFormatter(
    logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
)
logger.addHandler(handler)

file_handler = logging.FileHandler("info.log")
file_handler.setLevel(logging.INFO)
file_handler.setFormatter(
    logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
)
logger.addHandler(file_handler)



import asyncio
import logging
import sys
import time
from typing import Generic, TypeVar

from pydantic import BaseModel, Field

from dowse.exceptions import PreprocessorError
from dowse.interfaces import SourceT
from dowse.interfaces.executor import Executor
from dowse.interfaces.llms.classifier import Classifier
from dowse.interfaces.processor.base import Processor
from dowse.models.message import AgentMessage

logger = logging.getLogger("dowse")

T = TypeVar("T")
U = TypeVar("U")
Classifications = TypeVar("Classifications")


class Pipeline(BaseModel, Generic[T, Classifications]):
    """
    A pipeline is a sequence of preprocessors, a classifier, and a mapping of classifications to handlers.

    Args:
        T: The input type to the pipeline
        U: The intermediate type after preprocessing
        Classifications: The possible classifications from the classifier
        Response: The type of the response from the handlers
    """

    processors: list[Processor] = Field(default_factory=list)
    classifier: Classifier[U, Classifications]
    handlers: dict[Classifications, Executor]
    source: SourceT[T] | None = None

    async def run_processors(self, input_: T) -> AgentMessage[U]:
        if not self.processors:
            return AgentMessage(content=input_, error_message=None)

        for processor in self.processors:
            input_ = await processor.format(input_)
            if input_.is_error:
                raise PreprocessorError(input_.error_message)

        return input_

    async def process(self, input_: T) -> AgentMessage:
        processed_input = await self.run_processors(input_)

        logger.debug("Processed input: %s", processed_input)

        classification = await self.classifier.classify(processed_input.content)

        logger.debug("Classified as: %s", classification)

        response = await self.handlers[classification].execute(processed_input.content)

        logger.debug("Response: %s", response)

        return response

    async def run(
        self, max_executions: int = sys.maxsize, iteration_min_time: int = 0
    ) -> None:
        """A function to run the execution loop for the intent executor"""

        count = 0
        if self.source is None:
            raise ValueError("Source is not set")

        while count < max_executions:
            loop_start_time = time.time()
            data = await self.source.get_data()
            for item in data:
                logger.info("ITEM FOUND: %s", item)

                await self.process(item)

            count += 1

            loop_end_time = time.time()
            loop_time = loop_end_time - loop_start_time

            if loop_time < iteration_min_time:
                await asyncio.sleep(iteration_min_time - loop_time)



import logging
import os
from typing import Optional

import httpx

logger = logging.getLogger("dowse")

USER_ID_CACHE: dict[str, int] = {}


async def get_user_id(username: str) -> Optional[int]:
    """Convert a Twitter/X username to a user ID.

    Args:
        username: Twitter/X username (without @ symbol)

    Returns:
        User ID as integer if found, None if not found
    """
    # Remove @ if present
    username = username.lstrip("@")

    if username in USER_ID_CACHE:
        return USER_ID_CACHE[username]

    # Twitter API v2 endpoint
    url = f"https://api.twitter.com/2/users/by/username/{username}"

    # Get bearer token from environment
    bearer_token = os.environ.get("TWITTER_BEARER_TOKEN")
    if not bearer_token:
        raise ValueError("TWITTER_BEARER_TOKEN environment variable not set")

    headers = {"Authorization": f"Bearer {bearer_token}"}

    async with httpx.AsyncClient() as client:
        try:
            response = await client.get(url, headers=headers)
            response.raise_for_status()
            data = response.json()
            user_id = int(data["data"]["id"])
            USER_ID_CACHE[username] = user_id
            return user_id
        except (httpx.HTTPError, KeyError, ValueError) as e:
            logger.error(str(e))
            return None



import os

import httpx

from dowse.exceptions import TokenNotFoundError

from ..logger import logger


async def get_token_address(symbol: str, chain: str = "base") -> str:
    async with httpx.AsyncClient() as client:
        response = await client.get(
            f"https://api.dexscreener.com/latest/dex/search?q={symbol}&chain={chain}",
            timeout=20,
        )
    pairs = response.json()["pairs"]
    addresses = set()
    for pair in pairs:
        if pair["chainId"] != chain:
            continue
        quote = pair["quoteToken"]
        base = pair["baseToken"]

        if quote["symbol"] == symbol:
            addresses.add(quote["address"])
        elif base["symbol"] == symbol:
            addresses.add(base["address"])

    if len(addresses) == 0:
        raise TokenNotFoundError(f"Failed to find token for symbol: {symbol}")
    if len(addresses) > 1:
        raise TokenNotFoundError(
            f"Failed to find token for symbol: {symbol}, multiple existing tokens found: {addresses}"
        )
    return list(addresses)[0]


async def get_token_for_symbol(symbol: str) -> str:
    headers = {
        "accept": "application/json",
        "X-API-Key": os.environ["MORALIS_API_KEY"],
    }

    params = {
        "chain": "base",
        "symbols": [symbol],
    }

    tries = 0
    while tries < 3:
        tries += 1
        try:
            async with httpx.AsyncClient() as client:
                response = await client.get(
                    "https://deep-index.moralis.io/api/v2.2/erc20/metadata/symbols",
                    params=params,
                    headers=headers,
                    timeout=20,
                )
                data = response.json()
                return data[0]["address"]
        except Exception as e:
            logger.error("ERROR GETTING TOKEN FOR SYMBOL", e)

    raise ValueError("Failed to get token for symbol.  Most likely an API Limit")



import logging
from typing import Generic, TypeVar

from pydantic import BaseModel

from dowse.exceptions import PreprocessorError
from dowse.models.message import AgentMessage

from ..example_loader import ExampleLoader
from ..prompt_loader import PromptLoader
from .base import Processor

T = TypeVar("T")
U = TypeVar("U", bound=BaseModel)

logger = logging.getLogger("dowse")


class PreProcess(ExampleLoader, PromptLoader, Generic[T, U]):
    preprocessors: list[Processor]

    def __init_subclass__(cls, **kwargs):
        super().__init_subclass__(**kwargs)
        cls.load_examples()
        cls.load_prompt()

    async def run_preprocessors(self, input_: T) -> AgentMessage[U]:
        processed_data = input_
        if not self.preprocessors:
            return AgentMessage(content=processed_data, error_message=None)  # type: ignore[arg-type]

        for preprocessor in self.preprocessors:
            logger.debug("Running preprocessor: %s", processed_data)
            processed_data: AgentMessage = await preprocessor.format(processed_data)  # type: ignore[no-redef]
            logger.debug("Preprocessor result: %s", processed_data)
            if processed_data.error_message is not None:  # type: ignore[attr-defined]
                raise PreprocessorError(processed_data.error_message)  # type: ignore[attr-defined]
        return processed_data  # type: ignore



import os
from typing import Literal

import httpx
from eth_typing import HexAddress, HexStr
from pydantic import BaseModel

from dowse.logger import logger

QUICKNODE_ENDPOINT = os.environ["QUICKNODE_ENDPOINT"]


class Quote(BaseModel):
    token_in: HexAddress
    token_out: HexAddress
    amount_in: int
    amount_out: int
    gas: int
    aggregator: str = "kyber"


async def get_quote(
    token_in: HexAddress,
    token_out: HexAddress,
    amount: int,
    chain_id: Literal[1, 8453, 42161, 10, 137, 43114] = 8453,
) -> Quote:
    chain = get_chain_from_chain_id(chain_id)
    if token_in.lower() == "eth":
        token_in = HexAddress(HexStr("0x4200000000000000000000000000000000000006"))

    base_url = "https://aggregator-api.kyberswap.com"
    path = f"{chain}/api/v1/routes"
    params = {"tokenIn": token_in, "tokenOut": token_out, "amountIn": amount}
    url = f"{base_url}/{path}?tokenIn={params['tokenIn']}&tokenOut={params['tokenOut']}&amountIn={params['amountIn']}"

    tries = 0
    while tries < 3:
        tries += 1
        try:
            async with httpx.AsyncClient() as client:
                logger.debug(f"GET QUOTE: {token_in}, {token_out}, {amount}, {chain}")
                response = await client.get(url)
                response_json = response.json()
                data = response_json.get("data")
                route_summary = data.get("routeSummary")
                return Quote(
                    token_in=route_summary.get("tokenIn"),
                    token_out=route_summary.get("tokenOut"),
                    amount_in=route_summary.get("amountIn"),
                    amount_out=route_summary.get("amountOut"),
                    gas=route_summary.get("gas"),
                )
        except Exception as e:
            logger.error("ERROR GETTING QUOTE: %s", e)

    raise ValueError("Failed to get quote.  Most likely an API Limit")


def get_chain_from_chain_id(
    chain_id: Literal[1, 8453, 42161, 10, 137, 43114],
) -> Literal["ethereum", "base", "arbitrum", "optimism", "polygon", "avalanche"]:
    mapping: dict[
        int,
        Literal["ethereum", "base", "arbitrum", "optimism", "polygon", "avalanche"],
    ] = {
        8453: "base",
        42161: "arbitrum",
        10: "optimism",
        137: "polygon",
        43114: "avalanche",
        1: "ethereum",
    }
    return mapping[chain_id]



from typing import Awaitable

from pydantic import Field

from dowse.interfaces.effects import Effect
from dowse.models import Tweet

from ..llms.commands import CommandsList
from ..llms.questions import Response


class Printer(Effect[Tweet, CommandsList | Response]):
    prefix: str = Field(default="PRINTER")

    def execute(
        self,
        input_: Tweet,
        output: CommandsList | Response,
    ) -> Awaitable[None]:
        print(f"{self.prefix}:", output)
        return None



import logging
from abc import ABC
from typing import Any, Awaitable, Callable, Generic, TypeVar

from emp_agents import AgentBase, GenericTool
from emp_agents.models import Message, Provider
from emp_agents.providers import OpenAIModelType, OpenAIProvider
from pydantic import BaseModel, Field, PrivateAttr

from dowse.exceptions import PreprocessorError
from dowse.models.message import AgentMessage

from .effects import Effect
from .example_loader import ExampleLoader
from .processor import PreProcess, Processor
from .prompt_loader import PromptLoader

logger = logging.getLogger("dowse")

T = TypeVar("T", bound=BaseModel)
U = TypeVar("U", bound=BaseModel)
OutputType = TypeVar("OutputType")


class Executor(
    ABC, PreProcess[T, U], ExampleLoader, PromptLoader, Generic[T, U, OutputType]
):
    provider: Provider = Field(
        default_factory=lambda: OpenAIProvider(default_model=OpenAIModelType.gpt4o)
    )
    preprocessors: list[Processor] = Field(default_factory=list)
    tools: list[Callable | GenericTool] = Field(default_factory=list)
    effects: list[Effect[T, OutputType]] = Field(default_factory=list)
    examples: list[list[Message]] = Field(default_factory=list)

    _agent: AgentBase | None = PrivateAttr(default=None)

    def __init_subclass__(cls, **kwargs):
        super().__init_subclass__(**kwargs)
        cls.load_examples()
        cls.load_prompt()

    def model_post_init(self, __context: Any) -> None:
        super().model_post_init(__context)

        if self._prompt is not None:
            self.prompt = self._prompt
        if self._examples:
            self.examples = self._examples

    async def load_tools(self, input_: AgentMessage[U]) -> list[Callable | GenericTool]:
        """This can be overriden by subclasses to provide tools dynamically"""
        return self.tools

    async def after_execution(self, input_: T, output: OutputType) -> None:
        """A function that is called after the execution completes.
        Can be overridden by subclasses to add post-execution behavior.

        Args:
            input_: The original input that was executed
            output: The output from the execution
        """
        for effect in self.effects:
            maybe_coro = effect.execute(input_, output)
            if isinstance(maybe_coro, Awaitable):
                await maybe_coro

    async def execute(
        self,
        input_: T,
    ) -> AgentMessage[OutputType]:
        response_format_type = self._extract_response_format()
        response_format = AgentMessage[response_format_type]  # type: ignore[valid-type]
        response_format.__name__ = "AgentMessage"

        if response_format_type is None:
            return response_format(
                content=None,
                error_message="",
            )

        try:
            processed_input = await self.run_preprocessors(input_)
        except PreprocessorError as e:
            return response_format(
                content=None,
                error_message=str(e),
            )

        logger.debug("Processed input: (%s)", processed_input)

        if not self.prompt:
            raise ValueError("Prompt is required")

        self._agent = AgentBase(
            prompt=self.prompt,
            provider=self.provider,
            tools=await self.load_tools(processed_input),
        )

        assert processed_input.content is not None

        for example_lst in self.examples:
            self._agent.add_messages(example_lst)

        response = await self._agent.answer(
            processed_input.content.model_dump_json(),
            response_format=response_format,
        )

        formatted_response: AgentMessage[OutputType] = (  # type: ignore[valid-type]
            response_format.model_validate_json(response)
        )
        if formatted_response.content is None:
            logger.error(
                "No content in response: (%s) -> %s", processed_input, response
            )
            raise ValueError(formatted_response.error_message)
        await self.after_execution(input_, formatted_response.content)

        return formatted_response

    def _extract_response_format(self) -> type[OutputType]:
        if type(self).__base__ is Executor:
            return type(self).__pydantic_generic_metadata__["args"][2]
        return type(self).__bases__[0].__pydantic_generic_metadata__["args"][2]  # type: ignore[attr-defined]

    def __repr__(self):
        return f"{self.__class__.__name__}[{self._extract_response_format().__name__}]"

    __str__ = __repr__

    def __init__(self, **kwargs):
        if self._extract_response_format() == Any:
            raise ValueError("Response format is not set")
        super().__init__(**kwargs)

    def __rrshift__(self, other: Processor | list[Processor]) -> "Executor":
        if isinstance(other, list):
            self.preprocessors.extend(other)
        else:
            self.preprocessors.append(other)
        return self

    def __rshift__(
        self, other: Effect[T, OutputType] | list[Effect[T, OutputType]]
    ) -> "Executor":
        if isinstance(other, list):
            self.effects.extend(other)
        else:
            self.effects.append(other)
        return self



from pathlib import Path
from typing import ClassVar

from pydantic import BaseModel


class PromptLoader(BaseModel):
    prompt: str | None = None
    _path: ClassVar[Path]
    _prompt: ClassVar[str | None] = None

    @classmethod
    def load_prompt(cls) -> None:
        import inspect

        stack = inspect.stack()

        parent_path = Path(__file__).parent

        caller_frame = None
        for frame_info in stack:
            filename = frame_info.filename
            if filename.startswith("<frozen"):
                continue
            if Path(filename).is_relative_to(parent_path):
                continue
            if "pydantic" not in filename:
                caller_frame = frame_info
                break

        assert caller_frame is not None
        cls._path = Path(caller_frame.filename).parent

        prompt_path = cls._path / "PROMPT.txt"
        if prompt_path.exists():
            cls._prompt = prompt_path.read_text()



from abc import ABC, abstractmethod

from eth_typing import HexAddress
from pydantic import BaseModel


class UserManagerT(ABC, BaseModel):
    @abstractmethod
    async def get_user_address(self, user_id: str) -> HexAddress:
        user_address = await self.load_user_address(user_id)
        if user_address is None:
            user_address = await self.create_user_address(user_id)
        return user_address

    @abstractmethod
    async def create_user_address(self, user_id: str) -> HexAddress:
        pass

    @abstractmethod
    async def load_user_address(self, user_id: str) -> HexAddress | None:
        pass



from eth_typing import HexAddress

from dowse.interfaces import UserManagerT
from dowse.tools import get_user_address_helper


class BasicUserManager(UserManagerT):
    async def get_user_address(self, user_id: str) -> HexAddress:
        user_address = await self.load_user_address(user_id)
        if user_address is None:
            user_address = await self.create_user_address(user_id)
        return user_address

    @staticmethod
    async def create_user_address(user_id: str) -> HexAddress:
        raise ValueError("Not needed, addresses are deterministic")

    @staticmethod
    async def load_user_address(user_id: str) -> HexAddress | None:
        return await get_user_address_helper(user_id)



from eth_rpc import PrivateKeyWallet
from eth_rpc.types import primitives
from eth_typeshed import ERC20
from eth_typeshed.erc20 import TransferRequest
from eth_typing import HexAddress, HexStr


async def transfer(
    token_address: HexAddress,
    amount: int,
    recipient: HexAddress,
    wallet: PrivateKeyWallet,
) -> HexStr:
    contract = ERC20(address=token_address)
    tx_hash = await contract.transfer(
        TransferRequest(
            recipient=primitives.address(recipient),
            amount=primitives.uint256(amount),
        )
    ).execute(wallet)
    return tx_hash



from emp_agents.providers import OpenAIModelType, OpenAIProvider
from pydantic import BaseModel

from dowse.interfaces.executor import Executor
from dowse.models import Tweet

from ...tools import CommandRequest, Tools
from ..preprocessor import FormattedCommand, ProcessTokens
from .examples import EXAMPLES


class CommandsList(BaseModel):
    commands: list[CommandRequest]


BasicTwitterCommands = Executor[Tweet, FormattedCommand, CommandsList](
    prompt="",
    provider=OpenAIProvider(
        default_model=OpenAIModelType.gpt4o,
    ),
    preprocessors=[
        ProcessTokens(),
    ],
    tools=Tools.tools(),
    examples=EXAMPLES,
)



from abc import ABC
from typing import Any, Callable, Generic, TypeVar

from emp_agents import AgentBase
from emp_agents.models import Provider
from emp_agents.providers import OpenAIModelType, OpenAIProvider
from pydantic import BaseModel, Field

from dowse.interfaces.example_loader import ExampleLoader
from dowse.interfaces.prompt_loader import PromptLoader
from dowse.models.message import AgentMessage

T = TypeVar("T", bound=BaseModel)
U = TypeVar("U", bound=BaseModel)


class Processor(ABC, ExampleLoader, PromptLoader, Generic[T, U]):
    provider: Provider = OpenAIProvider(
        default_model=OpenAIModelType.gpt4o,
    )
    prompt: str = "Transform the input into the response format"
    tools: list[Callable] = Field(default_factory=list)

    def __init_subclass__(cls, **kwargs):
        super().__init_subclass__(**kwargs)
        cls.load_prompt()
        cls.load_examples()

    def model_post_init(self, __context: Any) -> None:
        super().model_post_init(__context)
        # Override the examples and prompt from the class variables
        if not self.examples:
            self.examples = self._examples
        if self._prompt is not None:
            self.prompt = self._prompt

    async def to_string(self, command: T) -> str:
        """This can be overriden by subclasses to modify the command before formatting"""
        return command.model_dump_json()

    async def get_tools(self, command: T) -> list[Callable]:
        """This can be overriden by subclasses to provide tools dynamically"""
        return self.tools

    async def format(self, command: T) -> AgentMessage[U]:
        command_string = await self.to_string(command)
        tools = await self.get_tools(command)
        return await self._format(command_string, tools)

    async def _format(self, command: str, tools: list[Callable]) -> AgentMessage[U]:
        agent = AgentBase(
            prompt=self.prompt,
            provider=self.provider,
            tools=tools,
        )

        for example in self.examples:
            agent.add_messages(example)

        response_content_type = self._extract_response_format()
        response_format = AgentMessage[response_content_type]  # type: ignore[valid-type]
        response_format.__name__ = "AgentMessage"

        response = await agent.answer(command, response_format=response_format)

        return AgentMessage[response_content_type].model_validate_json(response)  # type: ignore[valid-type]

    def _extract_response_format(self) -> type[U]:
        if type(self).__base__ is Processor:
            return type(self).__pydantic_generic_metadata__["args"][1]
        return type(self).__bases__[0].__pydantic_generic_metadata__["args"][1]  # type: ignore[attr-defined]



import httpx

from dowse.interfaces import ContextT


class BasicContextHelper(ContextT):
    context: str = ""

    async def load_extra_context(self) -> str:
        """loads the bitcoin whitepaper"""
        if self.context == "":
            async with httpx.AsyncClient() as client:
                whitepaper = await client.get(
                    "https://raw.githubusercontent.com/karask/satoshi-paper/refs/heads/master/bitcoin.md"
                )
            self.context = whitepaper.text
        return self.context



from .context import BasicContextHelper
from .effects import Printer
from .llms import (
    BasicTweetClassifier,
    BasicTwitterCommands,
    BasicTwitterQuestion,
    ProcessTokens,
)
from .source import TwitterMock
from .tools import CommandTools
from .user import BasicUserManager

__all__ = [
    "BasicUserManager",
    "BasicContextHelper",
    "CommandTools",
    "Printer",
    "BasicTweetClassifier",
    "BasicTwitterCommands",
    "BasicTwitterQuestion",
    "ProcessTokens",
    "TwitterMock",
]



import logging
from typing import Annotated

from eth_rpc.utils import to_checksum
from eth_typeshed.chainlink.eth_usd_feed import ChainlinkPriceOracle, ETHUSDPriceFeed
from eth_typing import ChecksumAddress
from typing_extensions import Doc

from dowse.exceptions import TokenNotFoundError

from .address import is_address
from .best_route.kyber import get_quote as get_kyber_quote
from .simulacrum import get_user_address, get_user_address_helper
from .symbol_to_address import get_token_address
from .twitter import get_user_id

logger = logging.getLogger("dowse")


async def get_token_address_tool(
    symbol_or_token_address: Annotated[
        str, Doc("The symbol of the token to get the address for")
    ],
) -> ChecksumAddress | str:
    """
    Get the address for a given token symbol
    """

    logger.debug("Tool Call: get_token_address (%s)", symbol_or_token_address)

    symbol_or_token_address = symbol_or_token_address.lstrip("$")

    if is_address(symbol_or_token_address):
        return to_checksum(symbol_or_token_address)
    if symbol_or_token_address.lower() == "eth":
        return to_checksum("0x4200000000000000000000000000000000000006")

    if symbol_or_token_address[0] == "@":
        user_id = await get_user_id(symbol_or_token_address)
        assert user_id is not None
        return await get_user_address(user_id)

    try:
        return to_checksum(await get_token_address(symbol_or_token_address))
    except TokenNotFoundError as e:
        return f"ERROR: {e}"


async def get_eth_price() -> float:
    """
    get the price of ETH in USD
    """

    eth_usd_feed = ETHUSDPriceFeed(address=ChainlinkPriceOracle.Ethereum.ETH)
    latest_round_data = await eth_usd_feed.latest_round_data().get()
    return latest_round_data.answer / 1e8


async def convert_dollar_amount_to_eth(
    amount: Annotated[str, Doc("The dollar amount to convert to ETH")],
) -> str:
    """
    Given a dollar amount, converts it to the corresponding amount in ETH.  For example, you could provide "$100"
    and it will strip the "$" and return an amount of ETH
    """
    eth_price = await get_eth_price()
    dollar_amount = float(amount.strip("$"))

    result = str(round(dollar_amount / eth_price, 4))

    logger.debug("Tool Call: convert_dollar_amount_to_eth (%s) -> %s", amount, result)
    return result


def convert_decimal_eth_to_wei(amount: str) -> str:
    """
    convert a decimal ETH amount to a string ETH amount
    """
    result = str(int(float(amount) * 1e18))
    logger.debug("Tool Call: convert_decimal_eth_to_wei (%s) -> %s", amount, result)
    return result


__all__ = [
    "get_token_address_tool",
    "convert_dollar_amount_to_eth",
    "convert_decimal_eth_to_wei",
    "get_kyber_quote",
    "is_address",
    "get_eth_price",
    "get_user_address_helper",
]



from typing import Literal

from dowse.interfaces.llms.classifier import Classifier
from dowse.models import Tweet

BasicTweetClassifier = Classifier[
    Tweet, Literal["commands", "question", "not_talking"]
]()



import os

import httpx
from eth_rpc import PrivateKeyWallet
from eth_typing import HexAddress, HexStr
from pydantic import BaseModel, Field

QUICKNODE_ENDPOINT = os.getenv("QUICKNODE_ENDPOINT")


class SwapResponse(BaseModel):
    from_: HexAddress = Field(alias="from")
    to: HexAddress
    data: str
    value: int
    chain_id: int = Field(alias="chainId")
    aggregator: str


async def execute_swap(
    token_in: HexAddress,
    token_out: HexAddress,
    amount: int,
    slippage: float,
    chain_id: int,
    recipient: HexAddress,
    aggregator: str,
    wallet: PrivateKeyWallet,
) -> HexStr:
    """Sends a raw swap from the quicknode endpoint and returns the tx hash"""

    url = f"{QUICKNODE_ENDPOINT}/addon/688/swap"
    async with httpx.AsyncClient() as client:
        response = await client.post(
            url,
            json={
                "aggregator": aggregator,
                "tokenIn": token_in,
                "tokenOut": token_out,
                "amountIn": amount,
                "slippage": slippage,
                "chainId": chain_id,
                "recipient": recipient,
            },
            headers={
                "Content-Type": "application/json",
            },
        )
    swap_data = SwapResponse(**response.json())

    signed_tx = wallet.prepare_and_sign(
        data=HexStr(swap_data.data),
        to=swap_data.to,
        value=swap_data.value,
    )

    tx_hash = await wallet.send_raw_transaction(signed_tx.raw_transaction)
    return tx_hash



from abc import ABC, abstractmethod
from asyncio import iscoroutinefunction
from typing import Awaitable, Generic, TypeVar

from pydantic import BaseModel

T = TypeVar("T")
U = TypeVar("U")


class Effect(ABC, BaseModel, Generic[T, U]):
    @abstractmethod
    def execute(self, input_: T, output: U) -> Awaitable[None] | None: ...

    async def __call__(self, input: T, output: U) -> Awaitable[None] | None:
        if iscoroutinefunction(self.execute):
            return await self.execute(input, output)
        return self.execute(input, output)



import math
from typing import Annotated

from eth_rpc import ContractFunc, ProtocolBase
from eth_rpc.networks import Base
from eth_rpc.types import METHOD, Name, primitives
from eth_rpc.utils import to_checksum
from eth_typing import ChecksumAddress
from typing_extensions import Doc

from .address import is_address
from .twitter import get_user_id

USER_ADDRESS_CACHE: dict[str, ChecksumAddress] = {}


def number_to_bytes(number):
    if number < 2**32:
        hex_string = f"{hex(number)[2:]:0>8}"
    else:
        nibble_count = int(math.log(number, 256)) + 1
        hex_string = "{:0{}x}".format(number, nibble_count * 2)
    return bytes.fromhex(hex_string)[:32]


class XSource(ProtocolBase):
    lookup_address: Annotated[
        ContractFunc[primitives.bytes32, primitives.address],
        Name("lookupAddress"),
    ] = METHOD


async def get_user_address(
    twitter_id: int,
) -> ChecksumAddress:
    twitter_id_bytes = number_to_bytes(twitter_id)
    xsource = XSource[Base](address="0x12854780b007d717ebc4905e906253fc549254a9")
    return to_checksum(await xsource.lookup_address(twitter_id_bytes).get())


async def get_user_address_helper(
    username: Annotated[str, Doc("The username to get the address for")],
) -> ChecksumAddress:
    """
    get the address for a given username
    """
    if is_address(username):
        return to_checksum(username)

    user_id = await get_user_id(username)
    assert user_id is not None
    if username not in USER_ADDRESS_CACHE:
        user_address = await get_user_address(user_id)
        USER_ADDRESS_CACHE[username] = user_address
    return USER_ADDRESS_CACHE[username]



# flake8: noqa: E501

from emp_agents.models import AssistantMessage, ToolCall, ToolMessage, UserMessage

EXAMPLES = [
    [
        UserMessage(
            content="{'caller': '@user', 'content': 'swap .01 ETH (0x4200000000000000000000000000000006) for $AERO (0x940181a94A35A4569E4529A3CDfB74e38FD98631)'}"
        ),
        AssistantMessage(
            content="let's convert the decimal to ETH and lookup the $AERO token address",
            tool_calls=[
                ToolCall(
                    id="aad543ba-f689-4101-9dfb-43d0f81f1a6f",
                    type="function",
                    function=ToolCall.Function(
                        name="convert_decimal_eth_to_wei",
                        arguments='{"amount": "0.01"}',
                    ),
                )
            ],
        ),
        ToolMessage(
            tool_call_id="aad543ba-f689-4101-9dfb-43d0f81f1a6f",
            content="10000000000000000",
        ),
        AssistantMessage(
            content="Let's use our swap tool to build this transaction",
            tool_calls=[
                ToolCall(
                    id="c4649732-4cc9-47e4-bc48-88224566686b",
                    type="function",
                    function=ToolCall.Function(
                        name="make_swap_command",
                        arguments='{"token_in": "0x4200000000000000000000000000000006", "token_out": "0x940181a94A35A4569E4529A3CDfB74e38FD98631", "amount": "10000000000000000", "recipient": "@user"}',
                    ),
                )
            ],
        ),
        ToolMessage(
            tool_call_id="c4649732-4cc9-47e4-bc48-88224566686b",
            content='{"amount_out": "1394601928424631358765", "aggregator": "Paraswap", "input_token_symbol": "$ETH", "input_token_address": "0x4200000000000000000000000000000006", "output_token": "$AERO", "output_token_address": "0x940181a94A35A4569E4529A3CDfB74e38FD98631"}',
        ),
        AssistantMessage(
            content="""
            {
                "commands": [
                    {"command": "swap", "args": {"token_in_address": "0x4200000000000000000000000000000006", "token_out_address": "0x940181a94A35A4569E4529A3CDfB74e38FD98631", "amount_in": "3000000000000000", "amount_out": "1394601928424631358765", "recipient": "@user"}},
                ]
            }
            """,
        ),
    ],
    [
        UserMessage(
            content="{'caller': '@user', 'content': 'swap for 0.003 ETH (0x4200000000000000000000000000000000000006) for $AIXBT (0x4F9Fd6Be4a90f2620860d680c0d4d5Fb53d1A825) and send half of it to @myfriend'}"
        ),
        AssistantMessage(
            content="Let's use our tools to convert the 0.003 ETH to an amount in wei",
            tool_calls=[
                ToolCall(
                    id="a6f8f1f1-aaeb-43e1-82a4-ad1cbf704fe1",
                    type="function",
                    function=ToolCall.Function(
                        name="convert_decimal_eth_to_wei",
                        arguments='{"amount": "0.003"}',
                    ),
                )
            ],
        ),
        ToolMessage(
            tool_call_id="a6f8f1f1-aaeb-43e1-82a4-ad1cbf704fe1",
            content="3000000000000000",
        ),
        AssistantMessage(
            content="Let's use our swap tool to build this transaction",
            tool_calls=[
                ToolCall(
                    id="c4649732-4cc9-47e4-bc48-88224566686b",
                    type="function",
                    function=ToolCall.Function(
                        name="make_swap_command",
                        arguments='{"token_in": "0x4200000000000000000000000000000000000006", "token_out": "0x4F9Fd6Be4a90f2620860d680c0d4d5Fb53d1A825", "amount": "3000000000000000", "recipient": "@user"}',
                    ),
                )
            ],
        ),
        ToolMessage(
            tool_call_id="c4649732-4cc9-47e4-bc48-88224566686b",
            content='{"amount_out": "1394601928424631358765", "aggregator": "Paraswap", "input_token_symbol": "ETH", "input_token_address": "0x4200000000000000000000000000000000000006", "output_token_symbol": "AIXBT", "output_token_address": "0x4F9Fd6Be4a90f2620860d680c0d4d5Fb53d1A825"}',
        ),
        AssistantMessage(
            content="Let's use our transfer tools to build this transaction",
            tool_calls=[
                ToolCall(
                    id="5bcd249f-45e6-415f-8582-bd32af4d84aa",
                    type="function",
                    function=ToolCall.Function(
                        name="make_transfer_command",
                        arguments='{"token": "0x4F9Fd6Be4a90f2620860d680c0d4d5Fb53d1A825", "amount": "697300964212315679382", "sender": "@user", "recipient": "@myfriend"}',
                    ),
                )
            ],
        ),
        ToolMessage(
            tool_call_id="5bcd249f-45e6-415f-8582-bd32af4d84aa",
            content='{"token_symbol": "AERO", "token_address": "0x4F9Fd6Be4a90f2620860d680c0d4d5Fb53d1A825", "sender": "@user", "recipient": "@myfriend", "value": "697300964212315679382"}',
        ),
        AssistantMessage(
            content="""
            {
                "commands": [
                    {"command": "swap", "args": {"token_in_address": "0x4200000000000000000000000000000006", "token_out_address": "0x4F9Fd6Be4a90f2620860d680c0d4d5Fb53d1A825", "amount_in": "3000000000000000", "amount_out": "1394601928424631358765", "recipient": "@user"}},
                    {"command": "transfer", "args": {"token_address": "0x4F9Fd6Be4a90f2620860d680c0d4d5Fb53d1A825", "amount": "697300964212315679382", "sender": "@user", "recipient": "@myfriend"}}
                ]
            }
            """,
        ),
    ],
    [
        UserMessage(
            content="{'caller': '@user', 'content': 'transfer 0.003 ETH (0x4200000000000000000000000000000000000006) to @myfriend'}"
        ),
        AssistantMessage(
            content="Let's use our transfer tools to build this transaction",
            tool_calls=[
                ToolCall(
                    id="2372fee1-1f62-47d6-835d-c0bc29f2236e",
                    type="function",
                    function=ToolCall.Function(
                        name="convert_decimal_eth_to_wei",
                        arguments='{"amount": "0.003"}',
                    ),
                ),
            ],
        ),
        ToolMessage(
            tool_call_id="2372fee1-1f62-47d6-835d-c0bc29f2236e",
            content="3000000000000000",
        ),
        AssistantMessage(
            content="Let's use our transfer tools to build this transaction",
            tool_calls=[
                ToolCall(
                    id="f06b6ed2-a09a-4456-9595-8cbfa9826c1e",
                    type="function",
                    function=ToolCall.Function(
                        name="make_transfer_command",
                        arguments='{"token": "0x4200000000000000000000000000000000000006", "amount": "3000000000000000", "sender": "@user", "recipient": "@myfriend"}',
                    ),
                )
            ],
        ),
        ToolMessage(
            tool_call_id="f06b6ed2-a09a-4456-9595-8cbfa9826c1e",
            content='{"token": "ETH", "token_address": "0x4200000000000000000000000000000000000006", "from": "@user", "to": "@myfriend", "value": "3000000000000000"}',
        ),
        AssistantMessage(
            content="""
            {
                "commands": [
                    {"command": "transfer", "args": {"token_address": "0x4200000000000000000000000000000000000006", "amount": "3000000000000000", "sender": "@user", "recipient": "@myfriend"}}
                ]
            }
            """,
        ),
    ],
]
